////////////////////////////////////////////////////////////////////////////////
//Description:  Digital trim crystal oscillator with 1s, 1m and 1h outputs     /
//Firmware:     0184-00-xx                                                     /
//Hardware:                                                                    /
//Processor:    PICxxxxx                                                       /
////////////////////////////////////////////////////////////////////////////////
//Revision history                                                             /
//No.   Date        Description                                                /
//-----------------------------------------------------------------------------/
//00    yyyy/mm/dd  Initial release                                            /
////////////////////////////////////////////////////////////////////////////////
#define     revision        0           //stored  in eeprom
#define     debugme         0           //0=standalone / 1=debug
#define     timing_diag1    0           //0=normal, 1=correction pulse on hours
#define     timing_diag2    1           //0=normal, 1=write constant to eeprom
////////////////////////////////////////////////////////////////////////////////
#include    "018400.h"
#include    "Divide32x8.c"
#include    "018400_memmap.c"
#include    "018400_interrupt.c"
////////////////////////////////////////////////////////////////////////////////






////////////////////////////////////////////////////////////////////////////////
void main(){
unsigned int8 pwm_counter;
unsigned int8 pwm_duty;
unsigned int1 down_flag;
unsigned int8 trim_byte;
unsigned int1 mod_flag;
unsigned int1 eeupdate_flag;
unsigned int1 zero_flag;
////////////////////////////////////////////////////////////////////////////////
mem_init();
////////////////////////////////////////////////////////////////////////////////
while(true){
restart_wdt();
//------------------------------------------------------------------------------

if (!reset_n)
    {
    disable_interrupts(global);
    //setup i/o for pwm out and trim control in
    adir=0b11111011;
    wpu =0b00000011;
    //compute pwm duty cycle
    trim_byte=read_eeprom(0);       //get current trim from eeprom...
    if (!(trim_byte & 0x80))        //...convert it to duty cycle
        pwm_duty=trim_byte+128;
    else
        pwm_duty=128-(trim_byte & 0x7f);
    //flag as not modified
    mod_flag=0;                     //1=trim has changed (EEPROM updated), 0=not
    down_flag=0;                    //1=a button was down
    eeupdate_flag=0;                //1=needs the eeprom written with new value
    //initialize pwm at zero
    pwm_counter=0;                  //when no buttons down, the current pwm cnt
    //output pwm
    while(!reset_n)
        {
        //----------------------------------------------------------------------
        //pwm update and wdt service
        restart_wdt();
        pwm_counter++;
        //pwm mode while reset_n low and no trim buttons pressed
        if ((apin & 3)==3)
            {
            down_flag=0;
            zero_flag=0;
            if (pwm_counter < pwm_duty)
                hoursout=1;
            else
                {
                hoursout=0;
                delay_cycles(1);
                }
            }
        //----------------------------------------------------------------------
        //adjustment mode while reset_n low and either button pressed
        else
            {
            //both buttons down?
            if ((!secondsout) && (!minutesout))
                {
                if (zero_flag==0)
                    {
                    zero_flag=1;
                    pwm_duty=128;
                    eeupdate_flag=1;
                    }
                }
            //only one button is down
            else
                {
                if (!down_flag)
                    {
                    if ((!secondsout) && (pwm_duty > 1))  //-127 min
                        {
                        eeupdate_flag=1;
                        pwm_duty--;
                        }
                    else
                        {
                        if ((!minutesout) && (pwm_duty < 255))
                            {
                            eeupdate_flag=1;
                            pwm_duty++;
                            }
                        }
                    }
                }
            down_flag=1;
            if (eeupdate_flag==1)
                {
                mod_flag=1;
                eeupdate_flag=0;
                if (pwm_duty>127)
                    write_eeprom(0,pwm_duty-128);
                else
                    write_eeprom(0,(128-pwm_duty) | 128);
                }
            }
        //----------------------------------------------------------------------
        }
    //if modified, compute new trim constant
    if (mod_flag)           //compute new only if changed
        trim_compute();
    //restart time counting
    reset_time();
    }

//------------------------------------------------------------------------------
}}
////////////////////////////////////////////////////////////////////////////////

