////////////////////////////////////////////////////////////////////////////////
//7        0                                                                   /
//xxxx xxHMS    portmask used to allow parallel write of clock bits            /
//-----------------------------------------------------------------------------/
//Each pin goes through one h-l-h cycle per its time unit.                     /
//Just like a ripple counter, all rising edges are aligned and the falling     /
//edges of the higher order outputs are aligned with the rising edges of the   /
//lower order outputs.  During reset, all outputs are high and when reset is   /
//deasserted, seconds, minutes and hours will go low 1/2 second, 30 seconds    /
//and 30 minutes later respectively.                                           /
//                                                                             /
// 1/2Sec      30Sec   1min         30min        1Hr                           /
// ||         |       |            |            |                              /
// _   _       _       _            _            _            _                /
//  |_| |_..._| |_..._| |_........_| |........._| |_........_| |_.......  Sec  /
// ___________         _____        _____        _____        _____            /
//            |_______|     |_...._|     |_...._|     |_...._|     |_...  Min  /
// ________________________________              ____________                  /
//                                 |____________|            |_______...  Hrs  /
// ^                                                                           /
// (Reset L-H)                                                                 /
////////////////////////////////////////////////////////////////////////////////
//#define     timebase    4                   //fast count test
#define     timebase    60                  //normal
////////////////////////////////////////////////////////////////////////////////
#int_TIMER0
void  TIMER0_isr(void){
//unsigned int1 do_trim=0;

#if (timing_diag1==1)
    apin=0;
#endif

if (trim_queue && trim_direction)
    {
    //if trim is in order and polarity is 1 (negative trim), then i throw away a
    //cycle to slow the clock down by a ratio of 1/trim_constant
    trim_queue=0;
    #if (timing_diag1==1)
        secondsout=1;
    #endif
    }
else
    {
    //only update port on 1/2 second edges, do this at the front edge of the int
    //when flagged by the time-keeping code
    if (update_time)
        {
        #if (timing_diag1==0)
            apin=time_pins;
        #else
            hoursout=1;
        #endif
        update_time=0;
        }
    //only do count ahead if it won't interfere with time keeping outputs, 
    //i.e. over clocking or under clocking pins.  if the prescaler happens to be
    //1 cycle before a time update i'll wait 31.25ms to do the adder trim
    if (trim_queue && (!trim_direction) && (prescaler != 15) && (prescaler !=31))
        {
        prescaler++;
        trim_queue=0;
        #if (timing_diag1==1)
            minutesout=1;
        #endif
        }
    //regular prescaler = 32 x 31.25ms = 500mS
    prescaler++;
    //ripple counter at overflow
    if (prescaler >= 32)
        {
        prescaler = 0;
        seconds++;
        if (seconds >= timebase)
            {
            seconds=0;
            minutes++;
            if (minutes>=timebase)
                {
                minutes=0;
                }
            }
        }
    //flag time update and build the pin image
    if ((prescaler==0) || (prescaler==16))
        {
        update_time=1;
        time_pins=0;
        if (prescaler == 0)                 //seconds pin comes from prescaler
            time_pins |= 1;
        if (seconds < (timebase/2))         //minutes pin comes from seconds
            time_pins |= 2;
        if (minutes < (timebase/2))         //hours pin comes from minutes
            time_pins |= 4;
        }
    }

//trim_direction is 0 or 1, count every single timer 0 overflow
//  0 in bit 7 is positive trim, i.e. slow clock
//  1 in bit 7 is negative trim, i.e. fast clock
if (trim_constant)
    {
    trim_counter--;
    if (trim_counter==0)
        {
        trim_counter=trim_constant;
        trim_queue=1;
        }
    }
}
////////////////////////////////////////////////////////////////////////////////
