////////////////////////////////////////////////////////////////////////////////
#byte       apin        =0x05
#bit        a0pin       =apin.0
#bit        a1pin       =apin.1
#bit        a2pin       =apin.2
#bit        a3pin       =apin.3
#bit        a4pin       =apin.4
#bit        a5pin       =apin.5
#byte       adir        =0x85
#bit        a0dir       =adir.0
#bit        a1dir       =adir.1
#bit        a2dir       =adir.2
//#bit      a3dir       =adir.3
#bit        a4dir       =adir.4
#bit        a5dir       =adir.5
#byte       option      =0x81
#bit        gppu_n      =option.7           //0=enabled, *1=disabled
#byte       wpu         =0x95               //xx54 x210: pull-ups, 1=on/0=off
//------------------------------------------------------------------------------
#define     reset_n     a3pin               //mclr is input only
#define     secondsout  a0pin
#define     minutesout  a1pin
#define     hoursout    a2pin
////////////////////////////////////////////////////////////////////////////////
#rom        0x2100      ={0x00}             //default is no trim
//          0x2110                          //store the computed count here!
#rom        0x2120      ={0,1,8,4,0,0,0,revision}  //FW id
////////////////////////////////////////////////////////////////////////////////
unsigned    int32       trim_constant;
unsigned    int32       trim_counter;
unsigned    int1        trim_direction;
unsigned    int1        trim_queue;
unsigned    int1        update_time;
unsigned    int8        prescaler;
unsigned    int8        seconds;
unsigned    int8        minutes;
unsigned    int8        time_pins;          //interrupt prebuilt time port value
////////////////////////////////////////////////////////////////////////////////
void reset_time (void){

//setup i/o for time outputs and all "1"
wpu =0b00000000;
apin=0b00000111;
adir=0b11111000;

//clear timer counters and start interrupt
prescaler=0;
seconds=0;
minutes=0;
set_timer0(0);
trim_counter=trim_constant;
update_time=0;                              //first pass through int displays
trim_queue=0;
enable_interrupts(global);
}
////////////////////////////////////////////////////////////////////////////////
void trim_compute (void){
unsigned int8 temp;

temp=read_eeprom(0);
if (!(temp & 0x7F))
    {
    trim_counter=0;
    trim_constant=0;
    trim_direction=0;
    }
else
    {
    if (temp & 0x80)
        trim_direction=1;
    else
        trim_direction=0;
    temp &= 0x7f;
    #if (timing_diag1==1)
        trim_constant=div32x8(16384,temp);
    #else
        trim_constant=div32x8(4000000,temp);
    #endif
    }
}
////////////////////////////////////////////////////////////////////////////////
void mem_init(void){

setup_wdt(WDT_2304MS);
setup_adc_ports(NO_ANALOGS|VSS_VDD);
setup_adc(ADC_OFF);
setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);    //16=0.5S, 2=0.0625S
setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);
setup_comparator(NC_NC);
setup_vref(FALSE);
enable_interrupts(int_timer0);
gppu_n=0;                                   //allow pull up control via wpu

//get trim value and compute constants
trim_compute();

//test computation
#if (timing_diag1 == 1) || (timing_diag2 == 1)
  write_eeprom(0x10,make8(trim_constant,3));
  write_eeprom(0x11,make8(trim_constant,2));
  write_eeprom(0x12,make8(trim_constant,1));
  write_eeprom(0x13,make8(trim_constant,0));
#endif

//initialize counts and i/o
reset_time();
}
////////////////////////////////////////////////////////////////////////////////

